<?php

namespace Drupal\flexible_weather_api\Plugin\WeatherApiLocationServicePlugin;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\flexible_weather_api\WeatherApiLocationServicePluginBase;

/**
 * The Google Geocode Api location service.
 *
 * @WeatherApiLocationServicePlugin(
 *   id="weather_api_location_service_google",
 *   label="Google (Geocode Api)",
 *   base_url="https://maps.googleapis.com/maps/api/geocode/json",
 * )
 */
class WeatherApiLocationServiceGoogle extends WeatherApiLocationServicePluginBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'service_options' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $config = $this->getConfiguration();
    $form['service_options']['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API key'),
      '#default_value' => isset($config['service_options']['api_key']) ? $config['service_options']['api_key'] : '',
      '#required' => TRUE,
      '#description' => $this->t('You can get API key on the <a href="@location-service-link">@location-service service</a>.', [
        '@location-service-link' => 'https://developers.google.com/maps/documentation/geocoding/get-api-key',
        '@location-service' => $this->t('Google (Geocoding API)'),
      ]),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->setConfiguration([
      'service_options' => [
        'api_key' => $form_state->getValue('api_key'),
      ],
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getAddress(string $text) {
    $languageManager = \Drupal::service('language_manager');
    $weather_api_requester = \Drupal::service('flexible_weather_api.weather_api_requester');
    $language_interface = $languageManager->getCurrentLanguage();
    $language = isset($language_interface) ? $language_interface->getId() : 'en';
    $config = $this->getConfiguration();
    $api_key = $config['service_options']['api_key'];
    $query = [
      'address' => $text,
      'language' => $language,
      'key' => $api_key,
    ];
    $service_options = [
      'api_key_required' => TRUE,
      'api_key' => $api_key,
      'query_params' => $query,
      'base_url' => $this->getBaseUrl(),
    ];
    $data = $weather_api_requester->retrieveDataFromCache($this->getPluginId(), $service_options, FALSE);
    $matches = [];
    if ($data) {
      foreach ($data->results as $result) {
        if (isset($result->address_components) && !empty($result->address_components)) {
          $formatted_address = '';
          foreach ($result->address_components as $component) {
            if (isset($component->types) && $component->types[0] == 'locality') {
              $formatted_address = $component->long_name;
              break;
            }
          }
          $matches[] = [
            'value' => Html::escape($formatted_address),
            'label' => Html::escape($formatted_address),
          ];
        }
      }
    }

    return $matches;
  }

}
